from typing import TYPE_CHECKING, Optional, List, Any

from InquirerPy.utils import InquirerPyDefault

if TYPE_CHECKING:
    from wexample_prompt.responses.interactive import ChoicePromptResponse


class ChoicePromptResponseManagerMixin:
    def choice(
        self,
        question: str,
        choices: List[Any],
        default: Optional[InquirerPyDefault] = None,
        abort: Optional[str] = "> Abort",
        **kwargs
    ) -> "ChoicePromptResponse":
        from wexample_prompt.responses.interactive import ChoicePromptResponse

        response = ChoicePromptResponse.create_choice(
            question=question,
            choices=choices,
            context=self.create_context(),
            default=default,
            abort=abort,
            **kwargs
        )

        self.print_response(response)
        return response



from abc import abstractmethod
from typing import TYPE_CHECKING, Optional

from pydantic import BaseModel

from wexample_prompt.mixins.with_io_manager import WithIoManager

if TYPE_CHECKING:
    from wexample_prompt.example.example_class_with_context import ExampleClassWithContext


class AbstractResponseExample(WithIoManager, BaseModel):
    class_with_context: Optional["ExampleClassWithContext"] = None

    def __init__(self, **kwargs):
        from wexample_prompt.common.io_manager import IoManager
        from wexample_prompt.example.example_class_with_context import ExampleClassWithContext

        super().__init__(**kwargs)

        self.io_manager = IoManager()
        self.class_with_context = ExampleClassWithContext(io_manager=self.io_manager)

    @abstractmethod
    def example_class(self, indentation: Optional[int] = None):
        pass

    @abstractmethod
    def example_manager(self):
        pass

    @abstractmethod
    def example_context(self):
        pass



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wexample_prompt.responses.titles.title_prompt_response import TitlePromptResponse


class TitlePromptResponseManagerMixin:
    """Mixin for IoManager to handle title responses."""

    def title(self, message: str, **kwargs) -> "TitlePromptResponse":
        from wexample_prompt.responses.titles.title_prompt_response import TitlePromptResponse

        """Create and display a title response."""
        response = TitlePromptResponse.create_title(
            text=message,
            context=self.create_context(),
        )

        if self._logger.handlers:
            self._logger.debug(message)

        self.print_response(response)
        return response



"""Mixin for managing progress prompt responses."""
from typing import Any, Optional, List

from wexample_prompt.responses.interactive.progress_prompt_response import ProgressPromptResponse
from wexample_prompt.progress.step_progress_context import ProgressStep


class ProgressPromptResponseManagerMixin:
    """Mixin class for managing progress prompt responses."""

    def progress(
        self,
        total: int,
        current: int,
        width: int = 50,
        label: Optional[str] = None,
        **kwargs: Any
    ) -> ProgressPromptResponse:
        """Create a progress bar.

        Args:
            total: Total number of items
            current: Current progress
            width: Width of the progress bar in characters
            label: Optional label to show above the progress bar
            **kwargs: Additional arguments passed to create_progress
        """
        response = ProgressPromptResponse.create_progress(
            total=total,
            current=current,
            width=width,
            label=label,
            context=self.create_context(),
            **kwargs
        )
        self.print_response(response)
        return response

    def progress_steps(
        self,
        steps: List[ProgressStep],
        width: int = 50,
        title: Optional[str] = None,
        **kwargs: Any
    ) -> ProgressPromptResponse:
        """Create a step-based progress bar.

        Args:
            steps: List of progress steps to execute
            width: Width of the progress bar
            title: Optional title for the progress bar
            **kwargs: Additional arguments passed to create_steps
        """
        context = ProgressPromptResponse.create_steps(
            steps=steps,
            width=width,
            title=title,
            context=self.create_context(),
            **kwargs
        )
        return context

    def progress_execute(
        self,
        callbacks: List[Any],
        width: int = 50,
        title: Optional[str] = None,
        **kwargs: Any
    ) -> List[Any]:
        """Execute a list of callbacks with progress tracking.

        Args:
            callbacks: List of callback functions to execute
            width: Width of the progress bar
            title: Optional title for the progress bar
            **kwargs: Additional arguments passed to execute
        """
        return ProgressPromptResponse.execute(
            callbacks=callbacks,
            width=width,
            title=title,
            context=self.create_context(),
            **kwargs
        )



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wexample_prompt.responses.messages import TaskPromptResponse


class TaskPromptResponseManagerMixin:
    def task(self, message: str, **kwargs) -> "TaskPromptResponse":
        from wexample_prompt.responses.messages import TaskPromptResponse

        response = TaskPromptResponse.create_task(
            message=message,
            context=self.create_context(),
        )

        self.print_response(response)
        return response



from typing import TYPE_CHECKING, Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.messages.debug_prompt_response import DebugPromptResponse

if TYPE_CHECKING:
    from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse

class DebugExample(AbstractResponseExample):

    def example_class(self, indentation: Optional[int] = None) -> "AbstractPromptResponse":
        """Example using the class directly."""
        return DebugPromptResponse.create_debug(
            'Test debug message',
            context=self.io_manager.create_context(indentation=indentation)
        )

    def example_manager(self):
        self.io_manager.debug('Test debug message')

    def example_context(self):
        self.class_with_context.debug('Test debug message')



from abc import abstractmethod
from typing import Optional, ClassVar

from wexample_prompt.common.color_manager import ColorManager
from wexample_prompt.enums.terminal_color import TerminalColor
from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.common.prompt_response_line import PromptResponseLine
from wexample_prompt.common.prompt_response_segment import PromptResponseSegment
from wexample_prompt.responses.base_prompt_response import BasePromptResponse

from wexample_prompt.common.prompt_context import PromptContext


class BaseMessageResponse(BasePromptResponse):
    # Symbol to display before the message, override in subclasses
    SYMBOL: ClassVar[str] = ""

    @classmethod
    @abstractmethod
    def get_message_type(cls) -> MessageType:
        pass

    @classmethod
    def _create_symbol_message(
        cls,
        text: str,
        color: TerminalColor,
        symbol: Optional[str] = None,
        bold_symbol: bool = True,
        context: Optional[PromptContext] = None,
        **kwargs
    ) -> 'BaseMessageResponse':
        """Create a message with a symbol.

        Args:
            text: Message text
            color: Text color
            symbol: Optional symbol to use instead of class symbol
            bold_symbol: Whether to make the symbol bold
            context: Optional prompt context
            **kwargs: Additional keyword arguments

        Returns:
            BaseMessageResponse: A new message response
        """
        segments = []

        # Add symbol if present
        if symbol or cls.SYMBOL:
            symbol_text = f"{symbol or cls.SYMBOL} "
            symbol_segment = PromptResponseSegment(
                text=ColorManager.colorize(
                    symbol_text,
                    color,
                    TerminalColor.BOLD if bold_symbol else None
                )
            )
            segments.append(symbol_segment)

        # Add message text
        message = PromptResponseSegment(
            text=ColorManager.colorize(text, color)
        )
        segments.append(message)

        # Create line with segments
        line = PromptResponseLine(
            segments=segments,
            line_type=cls.get_message_type()
        )

        # Create response with context and additional kwargs
        create_args = {"lines": [line], "message_type": cls.get_message_type()}
        if context is not None:
            if "params" in kwargs:
                context.params = kwargs.pop("params")
            create_args["context"] = context
        create_args.update(kwargs)

        return cls._create(**create_args)



from abc import abstractmethod
from typing import Protocol, TypeVar, TYPE_CHECKING, Dict, Any, Optional, List, Union

if TYPE_CHECKING:
    from wexample_prompt.responses.titles.title_prompt_response import TitlePromptResponse
    from wexample_prompt.responses.titles.subtitle_prompt_response import SubtitlePromptResponse
    from wexample_prompt.responses.messages.log_prompt_response import LogPromptResponse
    from wexample_prompt.responses.messages.info_prompt_response import InfoPromptResponse
    from wexample_prompt.responses.messages.debug_prompt_response import DebugPromptResponse
    from wexample_prompt.responses.messages.error_prompt_response import ErrorPromptResponse
    from wexample_prompt.responses.messages.failure_prompt_response import FailurePromptResponse
    from wexample_prompt.responses.messages.success_prompt_response import SuccessPromptResponse
    from wexample_prompt.responses.messages.task_prompt_response import TaskPromptResponse
    from wexample_prompt.responses.messages.warning_prompt_response import WarningPromptResponse
    from wexample_prompt.responses.data.list_prompt_response import ListPromptResponse
    from wexample_prompt.responses.data.table_prompt_response import TablePromptResponse
    from wexample_prompt.responses.data.tree_prompt_response import TreePromptResponse
    from wexample_prompt.responses.data.properties_prompt_response import PropertiesPromptResponse
    from wexample_prompt.responses.data.suggestions_prompt_response import SuggestionsPromptResponse
    from wexample_prompt.responses.interactive.progress_prompt_response import ProgressPromptResponse
    from wexample_prompt.responses.data.multiple_prompt_response import MultiplePromptResponse
    from wexample_prompt.responses.interactive.choice_prompt_response import ChoicePromptResponse
    from wexample_prompt.responses.interactive.choice_dict_prompt_response import ChoiceDictPromptResponse
    from wexample_prompt.responses.interactive.file_picker_prompt_response import FilePickerPromptResponse
    from wexample_prompt.responses.interactive.dir_picker_prompt_response import DirPickerPromptResponse

T = TypeVar('T', bound='BasePromptResponse')


class IoHandlerProtocol(Protocol):
    # Titles
    @abstractmethod
    def title(self, message: str, **kwargs) -> "TitlePromptResponse":
        pass

    @abstractmethod
    def subtitle(self, message: str, **kwargs) -> "SubtitlePromptResponse":
        pass

    # Messages
    @abstractmethod
    def log(self, message: str, **kwargs) -> "LogPromptResponse":
        pass

    @abstractmethod
    def info(self, message: str, **kwargs) -> "InfoPromptResponse":
        pass

    @abstractmethod
    def debug(self, message: str, **kwargs) -> "DebugPromptResponse":
        pass

    @abstractmethod
    def error(self, message: str, exception: Optional[Exception] = None, fatal: bool = True, **kwargs) -> "ErrorPromptResponse":
        pass

    @abstractmethod
    def failure(self, message: str, **kwargs) -> "FailurePromptResponse":
        pass

    @abstractmethod
    def success(self, message: str, **kwargs) -> "SuccessPromptResponse":
        pass

    @abstractmethod
    def task(self, message: str, **kwargs) -> "TaskPromptResponse":
        pass

    @abstractmethod
    def warning(self, message: str, **kwargs) -> "WarningPromptResponse":
        pass

    # Data display
    @abstractmethod
    def list(self, items: list, **kwargs) -> "ListPromptResponse":
        pass

    @abstractmethod
    def table(self, data: list, **kwargs) -> "TablePromptResponse":
        pass

    @abstractmethod
    def tree(self, data: Dict[str, Any], **kwargs) -> "TreePromptResponse":
        pass

    @abstractmethod
    def properties(self, properties: Dict[str, Any], **kwargs) -> "PropertiesPromptResponse":
        pass

    @abstractmethod
    def suggestions(self, suggestions: List[str], **kwargs) -> "SuggestionsPromptResponse":
        pass

    # Progress
    @abstractmethod
    def progress(self, total: int, message: Optional[str] = None, **kwargs) -> "ProgressPromptResponse":
        pass

    # Multiple responses
    @abstractmethod
    def multiple(self, responses: List[T], **kwargs) -> "MultiplePromptResponse":
        pass

    # Interactive
    @abstractmethod
    def choice(self, choices: List[str], message: Optional[str] = None, **kwargs) -> "ChoicePromptResponse":
        pass

    @abstractmethod
    def choice_dict(self, choices: Dict[str, Any], message: Optional[str] = None, **kwargs) -> "ChoiceDictPromptResponse":
        pass

    @abstractmethod
    def file_picker(self, path: str, pattern: Optional[str] = None, **kwargs) -> "FilePickerPromptResponse":
        pass

    @abstractmethod
    def dir_picker(self, path: str, **kwargs) -> "DirPickerPromptResponse":
        pass



"""Example for tree response."""
from typing import Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.data.tree_prompt_response import TreePromptResponse


class TreeExample(AbstractResponseExample):
    """Example for tree response."""

    def get_example(self) -> str:
        data = {
            "root": {
                "folder1": {
                    "file1": "content1",
                    "file2": "content2"
                },
                "folder2": {
                    "subfolder": {
                        "file3": "content3"
                    }
                }
            }
        }
        response = TreePromptResponse.create_tree(
            data=data,
            context=self.io_manager.create_context()
        )
        return response.render()

    def example_class(self, indentation: Optional[int] = None):
        """Example using class with context."""
        data = {
            "root": {
                "folder1": {
                    "file1": "content1",
                    "file2": "content2"
                },
                "folder2": {
                    "subfolder": {
                        "file3": "content3"
                    }
                }
            }
        }
        context = self.io_manager.create_context()
        if indentation is not None:
            context.indentation = indentation
        return TreePromptResponse.create_tree(
            data=data,
            context=context
        )

    def example_manager(self):
        """Example using IoManager directly."""
        data = {
            "root": {
                "folder1": {
                    "file1": "content1",
                    "file2": "content2"
                },
                "folder2": {
                    "subfolder": {
                        "file3": "content3"
                    }
                }
            }
        }
        self.io_manager.tree(data=data)

    def example_context(self):
        """Example using context."""
        data = {
            "root": {
                "folder1": {
                    "file1": "content1",
                    "file2": "content2"
                },
                "folder2": {
                    "subfolder": {
                        "file3": "content3"
                    }
                }
            }
        }
        self.class_with_context.tree(data=data)



from typing import ClassVar, Type, TYPE_CHECKING, Optional

from wexample_prompt.enums.terminal_color import TerminalColor
from wexample_prompt.responses.messages.base_message_response import BaseMessageResponse
from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.common.error_context import ErrorContext

if TYPE_CHECKING:
    from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse
    from wexample_prompt.example.abstract_response_example import AbstractResponseExample


class WarningPromptResponse(BaseMessageResponse):
    """Response for warning messages."""

    SYMBOL: ClassVar[str] = "⚠️"

    @classmethod
    def create_warning(
        cls: "WarningPromptResponse",
        message: str,
        context: ErrorContext = None,
        color: Optional[TerminalColor] = None,
        **kwargs
    ) -> "AbstractPromptResponse":
        return cls._create_symbol_message(
            text=message,
            context=context or ErrorContext(),
            color=color or TerminalColor.YELLOW,
            **kwargs
        )

    @classmethod
    def get_message_type(cls) -> MessageType:
        """Get the message type for warning messages."""
        return MessageType.WARNING

    @classmethod
    def get_example_class(cls) -> Type["AbstractResponseExample"]:
        """Get the example class for warning messages."""
        from wexample_prompt.example.response.messages.warning_example import WarningExample
        return WarningExample



"""Example usage of MultiplePromptResponse."""
from typing import List, Dict, Any, Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.data.list_prompt_response import ListPromptResponse
from wexample_prompt.responses.data.multiple_prompt_response import MultiplePromptResponse
from wexample_prompt.responses.messages.log_prompt_response import LogPromptResponse


class MultipleExample(AbstractResponseExample):
    """Example usage of MultiplePromptResponse."""

    def example_class(self, indentation: Optional[int] = None):
        """Example using the class directly."""
        responses = [
            LogPromptResponse.create_log("First response", context=self.io_manager.create_context()),
            ListPromptResponse.create_list(items=["Item 1", "Item 2"], context=self.io_manager.create_context()),
            LogPromptResponse.create_log("Last response", context=self.io_manager.create_context())
        ]
        return MultiplePromptResponse.create_multiple(
            responses=responses,
            context=self.io_manager.create_context(indentation=indentation)
        )

    def example_manager(self):
        """Example using the IoManager."""
        responses = [
            LogPromptResponse.create_log("First response", context=self.io_manager.create_context()),
            ListPromptResponse.create_list(items=["Item 1", "Item 2"], context=self.io_manager.create_context()),
            LogPromptResponse.create_log("Last response", context=self.io_manager.create_context())
        ]
        self.io_manager.multiple(responses=responses)

    def example_context(self):
        """Example using PromptContext."""
        responses = [
            LogPromptResponse.create_log("First response", context=self.io_manager.create_context()),
            ListPromptResponse.create_list(items=["Item 1", "Item 2"], context=self.io_manager.create_context()),
            LogPromptResponse.create_log("Last response", context=self.io_manager.create_context())
        ]
        self.class_with_context.multiple(responses=responses)

    def get_examples(self) -> List[Dict[str, Any]]:
        """Get list of examples.

        Returns:
            List[Dict[str, Any]]: List of example configurations
        """
        return [
            {
                "title": "Simple Multiple",
                "description": "Display multiple responses in sequence",
                "callback": self.simple_multiple
            },
            {
                "title": "Mixed Types",
                "description": "Display different types of responses together",
                "callback": self.mixed_types
            },
            {
                "title": "Dynamic Multiple",
                "description": "Build multiple responses dynamically",
                "callback": self.dynamic_multiple
            }
        ]

    def simple_multiple(self) -> Optional[MultiplePromptResponse]:
        """Show a simple multiple response example."""
        responses = [
            LogPromptResponse.create_log("First message", context=self.io_manager.create_context()),
            LogPromptResponse.create_log("Second message", context=self.io_manager.create_context()),
            LogPromptResponse.create_log("Third message", context=self.io_manager.create_context())
        ]
        return self.io_manager.multiple(responses=responses)

    def mixed_types(self) -> Optional[MultiplePromptResponse]:
        """Show different response types together."""
        responses = [
            LogPromptResponse.create_log("Log response", context=self.io_manager.create_context()),
            ListPromptResponse.create_list(
                items=["List item 1", "List item 2"],
                context=self.io_manager.create_context()
            ),
            LogPromptResponse.create_log("Another log response", context=self.io_manager.create_context())
        ]
        return self.io_manager.multiple(responses=responses)

    def dynamic_multiple(self) -> Optional[MultiplePromptResponse]:
        """Show building responses dynamically."""
        response = MultiplePromptResponse.create_multiple(
            responses=[LogPromptResponse.create_log("Initial response", context=self.io_manager.create_context())],
            context=self.io_manager.create_context()
        )

        # Add more responses
        response.append_response(
            ListPromptResponse.create_list(items=["Dynamic item 1"], context=self.io_manager.create_context())
        )
        response.extend_responses([
            LogPromptResponse.create_log("Added later", context=self.io_manager.create_context()),
            ListPromptResponse.create_list(items=["Dynamic item 2"], context=self.io_manager.create_context())
        ])

        return response



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wexample_prompt.responses.messages import InfoPromptResponse


class InfoPromptResponseManagerMixin:
    def info(self, message: str, **kwargs) -> "InfoPromptResponse":
        from wexample_prompt.responses.messages import InfoPromptResponse

        response = InfoPromptResponse.create_info(
            message=message,
            context=self.create_context(),
        )

        if self._logger.handlers:
            self._logger.debug(message)

        self.print_response(response)
        return response



"""Mixin for managing choice dict prompt responses."""
from typing import Any, Dict, Optional

from wexample_prompt.responses.interactive.choice_dict_prompt_response import ChoiceDictPromptResponse


class ChoiceDictPromptResponseManagerMixin:
    """Mixin class for managing choice dict prompt responses."""

    def choice_dict(
        self,
        question: str,
        choices: Dict[str, str],
        default: Optional[str] = None,
        abort: Optional[str] = "> Abort",
        **kwargs
    ) -> ChoiceDictPromptResponse:
        from wexample_prompt.responses.interactive import ChoiceDictPromptResponse

        response = ChoiceDictPromptResponse.create_choice_dict(
            question=question,
            choices=choices,
            context=self.create_context(),
            default=default,
            abort=abort,
            **kwargs
        )

        self.print_response(response)
        return response



"""Mixin for handling table responses in IoManager."""
from typing import List, Optional, Any, TYPE_CHECKING

from wexample_prompt.enums.verbosity_level import VerbosityLevel

if TYPE_CHECKING:
    from wexample_prompt.responses.data.table_prompt_response import TablePromptResponse


class TablePromptResponseManagerMixin:
    """Mixin for IoManager to handle table responses."""

    def table(
        self,
        data: List[List[Any]],
        headers: Optional[List[str]] = None,
        title: Optional[str] = None,
        **kwargs
    ) -> "TablePromptResponse":
        """Create and display a table response.

        Args:
            data: List of rows, each row being a list of values
            headers: Optional list of column headers
            title: Optional table title
            **kwargs: Additional arguments passed to create_table
        """
        from wexample_prompt.responses.data.table_prompt_response import TablePromptResponse

        response = TablePromptResponse.create_table(
            data=data,
            headers=headers,
            title=title,
            context=self.create_context(),
            **kwargs
        )

        self.print_response(response)
        return response



"""Example classes for message responses."""
from wexample_prompt.example.response.messages.debug_example import DebugExample
from wexample_prompt.example.response.messages.info_example import InfoExample
from wexample_prompt.example.response.messages.log_example import LogExample
from wexample_prompt.example.response.messages.success_example import SuccessExample

__all__ = [
    'DebugExample',
    'InfoExample',
    'LogExample',
    'SuccessExample',
]



from typing import Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.messages.warning_prompt_response import WarningPromptResponse


class WarningExample(AbstractResponseExample):

    def example_class(self, indentation: Optional[int] = None):
        return WarningPromptResponse.create_warning(
            'Test warning message',
            context=self.io_manager.create_context(indentation=indentation)
        )

    def example_manager(self):
        self.io_manager.warning('Test warning message')

    def example_context(self):
        self.class_with_context.warning('Test warning message')



"""Example usage of ProgressPromptResponse."""
import time
from typing import Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.progress.step_progress_context import ProgressStep
from wexample_prompt.responses.interactive.progress_prompt_response import ProgressPromptResponse


def step1() -> None:
    """First step: Initialize system."""
    time.sleep(0.5)


def step2() -> None:
    """Second step: Process data."""
    time.sleep(1)


def step3() -> None:
    """Third step: Cleanup."""
    time.sleep(0.5)


class ProgressExample(AbstractResponseExample):
    """Example usage of progress responses."""

    def example_class(self, indentation: Optional[int] = None):
        """Example using the response class directly."""
        return ProgressPromptResponse.create_progress(
            total=5,
            current=3,
            label="Processing",
            context=self.io_manager.create_context(indentation=indentation)
        )

    def example_manager(self):
        """Example using the IoManager."""
        self.io_manager.progress(
            total=5,
            current=3,
            label="Processing"
        )

    def example_context(self):
        """Example using a context class."""
        self.class_with_context.progress(
            total=5,
            current=3,
            label="Processing"
        )



"""Mixin for managing list prompt responses."""
from typing import List, Optional, Any

from wexample_prompt.responses.data.list_prompt_response import ListPromptResponse


class ListPromptResponseManagerMixin:
    """Mixin class for managing list prompt responses."""

    def list(
        self,
        items: List[str],
        bullet: str = "•",
        **kwargs: Any
    ) -> ListPromptResponse:
        """Create a list response.

        Args:
            items: List of items to display
            bullet: Bullet character to use (default: •)
            **kwargs: Additional arguments passed to create_list

        Returns:
            ListPromptResponse: A new list response
        """
        response = ListPromptResponse.create_list(
            items=items,
            bullet=bullet,
            context=self.create_context(),
            **kwargs
        )
        self.print_response(response)
        return response



from typing import Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.messages.task_prompt_response import TaskPromptResponse


class TaskExample(AbstractResponseExample):

    def example_class(self, indentation: Optional[int] = None):
        return TaskPromptResponse.create_task(
            'Test task message',
            context=self.io_manager.create_context(indentation=indentation)
        )

    def example_manager(self):
        self.io_manager.task('Test task message')

    def example_context(self):
        self.class_with_context.task('Test task message')



"""Abstract base class for interactive prompt responses."""
from abc import ABC

from wexample_prompt.responses.base_prompt_response import BasePromptResponse


class AbstractInteractivePromptResponse(BasePromptResponse, ABC):
    pass



"""Mixin for base prompt response."""
from typing import TYPE_CHECKING, List, Optional, Dict, Any, Union

from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.enums.response_type import ResponseType
from wexample_prompt.responses.base_prompt_response import BasePromptResponse

if TYPE_CHECKING:
    from wexample_prompt.common.prompt_response_line import PromptResponseLine


class BasePromptResponseManagerMixin:
    """Mixin for base prompt response."""

    def base(
        self,
        message: Union[str, List["PromptResponseLine"]],
        response_type: ResponseType = ResponseType.PLAIN,
        message_type: MessageType = MessageType.LOG,
        metadata: Optional[Dict[str, Any]] = None,
        **kwargs
    ) -> BasePromptResponse:
        """Create a base prompt response.

        Args:
            message: String message or list of response lines
            response_type: Response type
            message_type: Message type
            metadata: Optional metadata
            **kwargs: Additional keyword arguments

        Returns:
            BasePromptResponse: A new base prompt response
        """
        from wexample_prompt.common.prompt_response_line import PromptResponseLine
        from wexample_prompt.common.prompt_response_segment import PromptResponseSegment

        if isinstance(message, str):
            lines = [
                PromptResponseLine(segments=[
                    PromptResponseSegment(text=message)
                ])
            ]
        else:
            lines = message

        return BasePromptResponse.create_base(
            lines=lines,
            context=self.create_context(),
            response_type=response_type,
            message_type=message_type,
            metadata=metadata,
            **kwargs
        )



"""Example usage of FilePickerPromptResponse."""

from typing import Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.interactive.file_picker_prompt_response import FilePickerPromptResponse


class FilePickerExample(AbstractResponseExample):
    """Example usage of FilePickerPromptResponse."""

    def example_class(self, indentation: Optional[int] = None):
        """Example using the class directly."""
        return FilePickerPromptResponse.create_file_picker(
            question="Select a file:",
            base_dir=None,  # Will use current working directory
            context=self.io_manager.create_context(indentation=indentation)
        )

    def example_manager(self):
        """Example using the IoManager."""
        self.io_manager.file_picker("Select a file:")

    def example_context(self):
        """Example using PromptContext."""
        self.class_with_context.file_picker("Select a file:")



"""Mixin for managing directory picker prompt responses."""
from typing import Any, Optional

from wexample_prompt.responses.interactive.dir_picker_prompt_response import DirPickerPromptResponse


class DirPickerPromptResponseManagerMixin:
    """Mixin class for managing directory picker prompt responses."""

    def dir_picker(
        self,
        question: str = "Select a directory:",
        base_dir: Optional[str] = None,
        abort: Optional[str] = "> Abort",
        **kwargs: Any
    ) -> "DirPickerPromptResponse":
        """Create and execute a directory picker prompt.

        Args:
            question: The question to display
            base_dir: Starting directory (defaults to current working directory)
            abort: Optional abort choice text
            **kwargs: Additional arguments for inquirer.select

        Returns:
            str: Full path to selected directory, or None if aborted
        """
        response = DirPickerPromptResponse.create_dir_picker(
            question=question,
            base_dir=base_dir,
            abort=abort,
            **kwargs
        )
        self.print_response(response)
        return response



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wexample_prompt.responses.messages import WarningPromptResponse


class WarningPromptResponseManagerMixin:
    def warning(self, message: str, **kwargs) -> "WarningPromptResponse":
        from wexample_prompt.responses.messages import WarningPromptResponse

        response = WarningPromptResponse.create_warning(
            message=message,
            context=self.create_context(),
            **kwargs
        )

        self.print_response(response)
        return response



"""Example for properties response."""
from typing import Optional

from wexample_prompt.example.abstract_response_example import AbstractResponseExample
from wexample_prompt.responses.data.properties_prompt_response import PropertiesPromptResponse


class PropertiesExample(AbstractResponseExample):
    """Example for properties response."""

    def get_example(self) -> str:
        properties = {
            "name": "John Doe",
            "age": 30,
            "contact": {
                "email": "john@example.com",
                "phone": "123-456-7890"
            }
        }
        response = PropertiesPromptResponse.create_properties(
            properties=properties,
            title="User Information",
            context=self.io_manager.create_context()
        )
        return response.render()

    def example_class(self, indentation: Optional[int] = None):
        """Example using class with context."""
        properties = {
            "name": "John Doe",
            "age": 30,
            "contact": {
                "email": "john@example.com",
                "phone": "123-456-7890"
            }
        }
        context = self.io_manager.create_context()
        if indentation is not None:
            context.indentation = indentation
        return PropertiesPromptResponse.create_properties(
            properties=properties,
            title="User Information",
            context=context
        )

    def example_manager(self):
        """Example using IoManager directly."""
        properties = {
            "server": "localhost",
            "port": 8080,
            "config": {
                "debug": True,
                "log_level": "INFO"
            }
        }
        self.io_manager.properties(
            properties=properties,
            title="Server Configuration"
        )

    def example_context(self):
        """Example using context."""
        properties = {
            "status": "success",
            "duration": "2.5s",
            "details": {
                "processed": 100,
                "failed": 0
            }
        }
        self.class_with_context.properties(
            properties=properties,
            title="Operation Results"
        )



"""Mixin for handling tree responses in IoManager."""
from typing import Dict, Any, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from wexample_prompt.responses.data.tree_prompt_response import TreePromptResponse


class TreePromptResponseManagerMixin:
    """Mixin for IoManager to handle tree responses."""

    def tree(
        self,
        data: Dict[str, Any],
        **kwargs
    ) -> "TreePromptResponse":
        """Create and display a tree response.

        Args:
            data: Dictionary of hierarchical data to display
            **kwargs: Additional arguments passed to create_tree

        Returns:
            TreePromptResponse instance
        """
        from wexample_prompt.responses.data.tree_prompt_response import TreePromptResponse

        response = TreePromptResponse.create_tree(
            data=data,
            context=self.create_context(),
            **kwargs
        )

        self.print_response(response)
        return response



from typing import ClassVar, Optional, Any, Type, TYPE_CHECKING

from wexample_prompt.common.error_context import ErrorContext
from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.enums.terminal_color import TerminalColor
from wexample_prompt.responses import BasePromptResponse
from wexample_prompt.responses.data.multiple_prompt_response import MultiplePromptResponse
from wexample_prompt.responses.messages.base_message_response import BaseMessageResponse

if TYPE_CHECKING:
    from wexample_prompt.example.abstract_response_example import AbstractResponseExample


class ErrorPromptResponse(BaseMessageResponse):
    SYMBOL: ClassVar[str] = "❌"
    exception: Optional[Any] = None

    @classmethod
    def create_error(
        cls: "ErrorPromptResponse",
        message: Optional[str] = None,
        context: Optional[ErrorContext] = None,
        exception: Optional[Any] = None,
        color: Optional[TerminalColor] = None,
        **kwargs
    ) -> "ErrorPromptResponse":
        import traceback
        from wexample_prompt.enums.verbosity_level import VerbosityLevel

        # Build complete error message
        error_parts = []
        if message:
            error_parts.append(message)
        if exception:
            error_parts.append(str(exception))
            # Add traceback if there's an exception
            trace = ''.join(traceback.format_exception(type(exception), exception, exception.__traceback__))
            error_parts.append("\nTraceback:\n" + trace)

        # Create response with full error message
        response = cls._create_symbol_message(
            text="\n".join(error_parts) if error_parts else "Undefined error",
            color=color or TerminalColor.RED,
            context=context,
            **kwargs
        )

        response.exception = exception
        return response

    def is_fatal(self) -> bool:
        """Check if this is a fatal error."""
        if not isinstance(self.context, ErrorContext):
            return False
        return self.context.is_fatal

    def _on_fatal(self) -> None:
        """Handle fatal errors by raising the exception if present.

        Overrides BasePromptResponse._on_fatal()
        """
        # If we have an exception, raise it
        if self.context.fatal and self.exception:
            raise self.exception

    @classmethod
    def get_message_type(cls) -> MessageType:
        """Get the message type for error messages."""
        return MessageType.ERROR

    @classmethod
    def get_example_class(cls) -> Type["AbstractResponseExample"]:
        """Get the example class for error messages."""
        from wexample_prompt.example.response.messages.error_example import ErrorExample
        return ErrorExample



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from wexample_prompt.responses.titles.subtitle_prompt_response import SubtitlePromptResponse


class SubtitlePromptResponseManagerMixin:
    """Mixin for IoManager to handle subtitle responses."""

    def subtitle(self, message: str, **kwargs) -> "SubtitlePromptResponse":
        from wexample_prompt.responses.titles.subtitle_prompt_response import SubtitlePromptResponse
        
        """Create and display a subtitle response."""
        response = SubtitlePromptResponse.create_subtitle(
            text=message,
            context=self.create_context(),
        )

        if self._logger.handlers:
            self._logger.debug(message)

        self.print_response(response)
        return response
